#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <SDL.h>

#include "gp2x.h"
#include "mmsp2adc.h"

SDL_Color scwhite = {0xFF,0xFF,0xFF};
SDL_Color scgrey  = {0x77,0x77,0x77};
SDL_Color scblack = {0x00,0x00,0x00};
SDL_Color scballs[] = {
    {0x00,0x00,0xFF},
    {0xFF,0x00,0x00},
    {0x00,0x00,0x00},
    {0xFF,0xFF,0x00},
    {0x00,0xA5,0x00},
    {0xFF,0xA5,0x00}
};
Uint32 white, grey, black, balls[36];
int ballc = 6;

unsigned char deadness[] = {
    0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0
};

// display the deadness board
void display_deadness(SDL_Surface *screen);

// mark a point on the deadness board
void mark_deadness(SDL_Surface *screen, int px, int py);

int main()
{
    SDL_Surface *screen;
    unsigned int buttonState;
    int i;

    // initialize SDL
    if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0) {
        fprintf(stderr, "Failed to initialize SDL: %s\n", SDL_GetError());
        exit(1);
    }

    atexit(SDL_Quit);

    screen = SDL_SetVideoMode (320, 240, 16, SDL_SWSURFACE);
    if (screen == NULL) {
        fprintf(stderr, "Failed to initialize SDL video: %s\n", SDL_GetError());
        exit(1);
    }

    // make sure all the modes are set properly
    SDL_ShowCursor(0);
    SDL_JoystickOpen(0);
    SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);

    buttonState = 0;
    
    // copy colors over the right format
    white = SDL_MapRGB(screen->format, scwhite.r, scwhite.g, scwhite.b);
    grey  = SDL_MapRGB(screen->format, scgrey.r,  scgrey.g,  scgrey.b);
    black = SDL_MapRGB(screen->format, scblack.r, scblack.g, scblack.b);
    for (i = 0; i < ballc*ballc; i++) {
        balls[i] = SDL_MapRGB(screen->format, scballs[i].r, scballs[i].g, scballs[i].b);
    }

    /* Now do the actual iteration */
    while (1) {
        SDL_Event event;

        display_deadness(screen);

        if (SDL_WaitEvent(&event)) {
            switch (event.type) {
                case SDL_QUIT:
                {
                    exit(0);
                }

                case SDL_JOYBUTTONDOWN:
                {
                    buttonState |= (1<<event.jbutton.button);

                    if (buttonState == ((1<<GP2X_BUTTON_START)|(1<<GP2X_BUTTON_SELECT))) {
                        // reset the deadness board
                        memset(deadness, 0, ballc*ballc);

                    } else if (buttonState == ((1<<GP2X_BUTTON_L)|(1<<GP2X_BUTTON_R))) {
                        // quit
                        exit(0);

                    }
                    break;
                }

                case SDL_JOYBUTTONUP:
                {
                    buttonState &= ~(1<<event.jbutton.button);
                    break;
                }

                case SDL_MOUSEBUTTONDOWN:
                {
                    // mark this deadness
                    mark_deadness(screen, event.button.x, event.button.y);
                    break;
                }
            }
        }
    }

    return 0;
}

void display_deadness(SDL_Surface *screen)
{
    // necessary constants
    int cellw = screen->w / (ballc+1);
    int cellh = screen->h / (ballc+1);
    int i, x, y;
    SDL_Rect cell;

    // start blank
    SDL_FillRect(screen, NULL, white);

    // draw the borders for clarity
    cell.w = 1;
    cell.h = screen->h;
    cell.y = 0;
    for (i = 1; i <= ballc + 1; i++) {
        cell.x = i * cellw;
        if (cell.x < screen->w)
            SDL_FillRect(screen, &cell, black);
    }

    cell.w = screen->w;
    cell.h = 1;
    cell.x = 0;
    for (i = 1; i <= ballc + 1; i++) {
        cell.y = i * cellh;
        if (cell.y < screen->h)
            SDL_FillRect(screen, &cell, black);
    }

    // grey out the center
    cell.w = cellw - 1;
    cell.h = cellh - 1;
    for (i = 1; i <= ballc; i++) {
        cell.x = i * cellw + 1;
        cell.y = i * cellh + 1;
        SDL_FillRect(screen, &cell, grey);
    }

    // draw the battery level
    cell.x = 5;
    cell.y = 5;
    cell.w = (4 - mmsp2adc()) * (cellw-10) / 4;
    if (cell.w > cellw-10) cell.w = cellw-10;
    cell.h = (cellh-10) / 4;
    SDL_FillRect(screen, &cell, balls[1]);

    // draw the border cells
    cell.w = cellw - 10;
    cell.h = cellh - 10;
    for (i = 0; i < ballc; i++) {
        cell.x = cellw*(i+1) + 5;
        cell.y = 5;
        SDL_FillRect(screen, &cell, balls[i]);
    }
    for (i = 0; i < ballc; i++) {
        cell.x = 5;
        cell.y = cellh*(i+1) + 5;
        SDL_FillRect(screen, &cell, balls[i]);
    }

    // draw the deadness itself
    cell.w /= 2;
    for (y = 0; y < ballc; y++) {
        for (x = 0; x < ballc; x++) {
            if (x == y) continue;
            if (deadness[y*ballc + x]) {
                cell.x = cellw*(x+1) + 5;
                cell.y = cellh*(y+1) + 5;
                SDL_FillRect(screen, &cell, balls[y]);
                cell.x += cell.w;
                SDL_FillRect(screen, &cell, balls[x]);
            }
        }
    }

    SDL_UpdateRect(screen, 0, 0, screen->w, screen->h);
}

void mark_deadness(SDL_Surface *screen, int px, int py)
{
    // necessary constants
    int cellw = screen->w / (ballc+1);
    int cellh = screen->h / (ballc+1);
    int i, x, y;

    // now convert the pixel locations into an x and y
    x = (px / cellw) - 1;
    y = (py / cellh) - 1;

    // if y is invalid, this makes no sense
    if (y < 0 || y >= ballc) return;

    // if x is -1, clear an entire row
    if (x < 0) {
        for (x = 0; x < ballc; x++) {
            deadness[y*ballc + x] = 0;
        }
        return;

    } else if (x >= ballc) {
        return;
        
    }

    // otherwise, just swap the deadness
    i = y*ballc + x;
    deadness[i] = !deadness[i];
}
